@@ -14,19 +14,15 @@
  * Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
-#include <pango/pango.h>
-#include <cairo.h>
-
 #include <mousepad/mousepad-private.h>
-#include <mousepad/mousepad-preferences.h>
+#include <mousepad/mousepad-gtkcompat.h>
 #include <mousepad/mousepad-document.h>
 #include <mousepad/mousepad-util.h>
 #include <mousepad/mousepad-print.h>
 
+#include <gtksourceview/gtksourcebuffer.h>
+#include <gtksourceview/gtksourceprintcompositor.h>
+
 #define DOCUMENT_SPACING (10)
 
 
@@ -90,7 +86,7 @@ struct _MousepadPrint
 
 
 
-G_DEFINE_TYPE (MousepadPrint, mousepad_print, GTK_TYPE_PRINT_OPERATION);
+G_DEFINE_TYPE (MousepadPrint, mousepad_print, GTK_TYPE_PRINT_OPERATION)
 
 
 
@@ -159,8 +155,8 @@ mousepad_print_settings_load (GtkPrintOperation *operation)
   PangoContext          *context;
   PangoFontDescription  *font_desc;
 
-  mousepad_return_if_fail (MOUSEPAD_IS_DOCUMENT (print->document));
-  mousepad_return_if_fail (GTK_IS_WIDGET (print->document->textview));
+  g_return_if_fail (MOUSEPAD_IS_DOCUMENT (print->document));
+  g_return_if_fail (GTK_IS_WIDGET (print->document->textview));
 
   /* get the config file filename */
   filename = mousepad_util_get_save_location (MOUSEPAD_RC_RELPATH, FALSE);
@@ -286,6 +282,12 @@ mousepad_print_settings_load (GtkPrintOperation *operation)
     else
       gtk_source_print_compositor_set_line_numbers_font_name (print->compositor, body_font);
 
+    /* setup line number printing */
+    if (print->print_line_numbers)
+      gtk_source_print_compositor_set_print_line_numbers (print->compositor, print->line_number_increment);
+    else
+      gtk_source_print_compositor_set_print_line_numbers (print->compositor, 0);
+
     /* cleanup */
     g_free (body_font);
     g_free (header_font);
@@ -502,6 +504,10 @@ mousepad_print_button_toggled (GtkWidget     *button,
   {
     print->print_line_numbers = active;
     gtk_widget_set_sensitive (print->widget_line_numbers_hbox, active);
+    if (active)
+      gtk_source_print_compositor_set_print_line_numbers (print->compositor, print->line_number_increment);
+    else
+      gtk_source_print_compositor_set_print_line_numbers (print->compositor, 0);
   }
   else if (button == print->widget_text_wrapping)
     gtk_source_print_compositor_set_wrap_mode (print->compositor, active ? GTK_WRAP_WORD : GTK_WRAP_NONE);
@@ -536,13 +542,8 @@ mousepad_print_spin_value_changed (GtkSpinButton *button,
 {
   print->line_number_increment = gtk_spin_button_get_value_as_int (button);
 
-  if (print->line_number_increment > 0 && print->print_line_numbers)
-    {
-      gtk_source_print_compositor_set_print_line_numbers (print->compositor,
-                                                          print->line_number_increment);
-    }
-  else
-    gtk_source_print_compositor_set_print_line_numbers (print->compositor, 0);
+  gtk_source_print_compositor_set_print_line_numbers (print->compositor,
+                                                      print->line_number_increment);
 }
 
 
@@ -584,7 +585,7 @@ mousepad_print_create_custom_widget (GtkPrintOperation *operation)
   GtkWidget     *table;
   GtkAdjustment *adjustment;
 
-  vbox = gtk_vbox_new (FALSE, 6);
+  vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 6);
   gtk_container_set_border_width (GTK_CONTAINER (vbox), 8);
 
   frame = gtk_frame_new (NULL);
@@ -622,7 +623,7 @@ mousepad_print_create_custom_widget (GtkPrintOperation *operation)
   gtk_container_add (GTK_CONTAINER (frame), alignment);
   gtk_widget_show (alignment);
 
-  vbox2 = gtk_vbox_new (FALSE, 6);
+  vbox2 = gtk_box_new (GTK_ORIENTATION_VERTICAL, 6);
   gtk_container_add (GTK_CONTAINER (alignment), vbox2);
   gtk_widget_show (vbox2);
 
@@ -645,18 +646,22 @@ mousepad_print_create_custom_widget (GtkPrintOperation *operation)
   gtk_box_pack_start (GTK_BOX (vbox2), alignment, FALSE, FALSE, 0);
   gtk_widget_show (alignment);
 
-  print->widget_line_numbers_hbox = gtk_hbox_new (FALSE, 6);
+  print->widget_line_numbers_hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 6);
   gtk_widget_set_sensitive (print->widget_line_numbers_hbox, print->print_line_numbers);
   gtk_container_add (GTK_CONTAINER (alignment), print->widget_line_numbers_hbox);
   gtk_widget_show (print->widget_line_numbers_hbox);
 
-  label = gtk_label_new (_("Number every"));
+  label = gtk_label_new (_("Numbering interval:"));
   gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
   gtk_box_pack_start (GTK_BOX (print->widget_line_numbers_hbox), label, FALSE, TRUE, 0);
   gtk_widget_show (label);
 
   adjustment = GTK_ADJUSTMENT (gtk_adjustment_new (1.0, 1.0, 100.0, 1.0, 0.0, 0.0));
   print->widget_line_numbers_spin = gtk_spin_button_new (adjustment, 1.0, 0);
+  gtk_widget_set_tooltip_text (print->widget_line_numbers_spin,
+    _("The interval at which to print line numbers. For example a value of 1 "
+      "will print a line number on each line, a value of 2 will print a line "
+      "number on every other line, and so on."));
   gtk_spin_button_set_value (GTK_SPIN_BUTTON (print->widget_line_numbers_spin),
                              (gdouble) print->line_number_increment);
   g_signal_connect (G_OBJECT (print->widget_line_numbers_spin),
@@ -666,11 +671,6 @@ mousepad_print_create_custom_widget (GtkPrintOperation *operation)
   gtk_box_pack_start (GTK_BOX (print->widget_line_numbers_hbox), print->widget_line_numbers_spin, FALSE, TRUE, 0);
   gtk_widget_show (print->widget_line_numbers_spin);
 
-  label = gtk_label_new (_("line(s)"));
-  gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
-  gtk_box_pack_start (GTK_BOX (print->widget_line_numbers_hbox), label, FALSE, TRUE, 0);
-  gtk_widget_show (label);
-
   button = print->widget_text_wrapping = gtk_check_button_new_with_mnemonic (_("Enable text _wrapping"));
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button),
                                 gtk_source_print_compositor_get_wrap_mode (print->compositor) == GTK_WRAP_NONE ? FALSE : TRUE);
@@ -700,6 +700,12 @@ mousepad_print_create_custom_widget (GtkPrintOperation *operation)
   gtk_container_add (GTK_CONTAINER (frame), alignment);
   gtk_widget_show (alignment);
 
+  /* In GTK3, GtkTable is deprecated */
+#if GTK_CHECK_VERSION(3, 0, 0) && (__GNUC__ > 4 || __GNUC__ == 4 && __GNUC_MINOR__ > 2)
+# pragma GCC diagnostic push
+# pragma GCC diagnostic ignored "-Wdeprecated-declarations"
+#endif
+
   table = gtk_table_new (3, 2, FALSE);
   gtk_table_set_col_spacings (GTK_TABLE (table), 6);
   gtk_table_set_row_spacings (GTK_TABLE (table), 6);
@@ -736,6 +742,10 @@ mousepad_print_create_custom_widget (GtkPrintOperation *operation)
   g_signal_connect (G_OBJECT (print->widget_line_numbers_font), "font-set", G_CALLBACK (mousepad_print_button_font_set), print);
   gtk_widget_show (print->widget_line_numbers_font);
 
+#if GTK_CHECK_VERSION(3, 0, 0) && (__GNUC__ > 4 || __GNUC__ == 4 && __GNUC_MINOR__ > 2)
+# pragma GCC diagnostic pop
+#endif
+
   return vbox;
 }
 
@@ -779,12 +789,12 @@ mousepad_print_document_interactive (MousepadPrint     *print,
 {
   GtkPrintOperationResult result;
 
-  mousepad_return_val_if_fail (MOUSEPAD_IS_PRINT (print), FALSE);
-  mousepad_return_val_if_fail (GTK_IS_PRINT_OPERATION (print), FALSE);
-  mousepad_return_val_if_fail (MOUSEPAD_IS_DOCUMENT (document), FALSE);
-  mousepad_return_val_if_fail (GTK_IS_SOURCE_BUFFER (document->buffer), FALSE);
-  mousepad_return_val_if_fail (GTK_IS_WINDOW (parent), FALSE);
-  mousepad_return_val_if_fail (error == NULL || *error == NULL, FALSE);
+  g_return_val_if_fail (MOUSEPAD_IS_PRINT (print), FALSE);
+  g_return_val_if_fail (GTK_IS_PRINT_OPERATION (print), FALSE);
+  g_return_val_if_fail (MOUSEPAD_IS_DOCUMENT (document), FALSE);
+  g_return_val_if_fail (GTK_SOURCE_IS_BUFFER (document->buffer), FALSE);
+  g_return_val_if_fail (GTK_IS_WINDOW (parent), FALSE);
+  g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
 
   /* set the document */
   print->document = document;
