@@ -17,6 +17,9 @@
 #ifndef __MOUSEPAD_STATUSBAR_H__
 #define __MOUSEPAD_STATUSBAR_H__
 
+#include <gtk/gtk.h>
+#include <gtksourceview/gtksourcelanguage.h>
+
 G_BEGIN_DECLS
 
 #define MOUSEPAD_TYPE_STATUSBAR            (mousepad_statusbar_get_type ())
@@ -44,6 +47,12 @@ void        mousepad_statusbar_set_overwrite        (MousepadStatusbar *statusba
 void        mousepad_statusbar_set_language         (MousepadStatusbar *statusbar,
                                                      GtkSourceLanguage *language);
 
+gboolean    mousepad_statusbar_push_tooltip         (MousepadStatusbar *statusbar,
+                                                     GtkWidget         *widget);
+
+void        mousepad_statusbar_pop_tooltip          (MousepadStatusbar *statusbar,
+                                                     GtkWidget         *widget);
+
 G_END_DECLS
 
 #endif /* !__MOUSEPAD_STATUSBAR_H__ */
