@@ -14,15 +14,8 @@
  * Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
-#ifdef HAVE_STRING_H
-#include <string.h>
-#endif
-
 #include <mousepad/mousepad-private.h>
+#include <mousepad/mousepad-gtkcompat.h>
 #include <mousepad/mousepad-util.h>
 
 
@@ -149,7 +142,7 @@ mousepad_util_iter_backward_word_start (GtkTextIter *iter)
 gboolean
 mousepad_util_iter_forward_text_start (GtkTextIter *iter)
 {
-  mousepad_return_val_if_fail (!mousepad_util_iter_inside_word (iter), FALSE);
+  g_return_val_if_fail (!mousepad_util_iter_inside_word (iter), FALSE);
 
   /* keep until we hit text or a line end */
   while (g_unichar_isspace (gtk_text_iter_get_char (iter)))
@@ -166,7 +159,7 @@ mousepad_util_iter_backward_text_start (GtkTextIter *iter)
 {
   GtkTextIter prev = *iter;
 
-  mousepad_return_val_if_fail (!mousepad_util_iter_inside_word (iter), FALSE);
+  g_return_val_if_fail (!mousepad_util_iter_inside_word (iter), FALSE);
 
   while (!gtk_text_iter_starts_line (&prev) && gtk_text_iter_backward_char (&prev))
     {
@@ -252,7 +245,7 @@ mousepad_util_utf8_strcapital (const gchar *str)
   GString     *result;
   gboolean     upper = TRUE;
 
-  mousepad_return_val_if_fail (g_utf8_validate (str, -1, NULL), NULL);
+  g_return_val_if_fail (g_utf8_validate (str, -1, NULL), NULL);
 
   /* create a new string */
   result = g_string_sized_new (strlen (str));
@@ -307,7 +300,7 @@ mousepad_util_utf8_stropposite (const gchar *str)
   gchar       *buf;
   GString     *result;
 
-  mousepad_return_val_if_fail (g_utf8_validate (str, -1, NULL), NULL);
+  g_return_val_if_fail (g_utf8_validate (str, -1, NULL), NULL);
 
   /* create a new string */
   result = g_string_sized_new (strlen (str));
@@ -393,7 +386,7 @@ mousepad_util_entry_error (GtkWidget *widget,
   const GdkColor white = {0, 0xffff, 0xffff, 0xffff};
   gpointer       pointer;
 
-  mousepad_return_if_fail (GTK_IS_WIDGET (widget));
+  g_return_if_fail (GTK_IS_WIDGET (widget));
 
   /* get the current error state */
   pointer = mousepad_object_get_data (G_OBJECT (widget), "error-state");
@@ -421,24 +414,28 @@ mousepad_util_dialog_header (GtkDialog   *dialog,
   gchar     *full_title;
   GtkWidget *vbox, *ebox, *hbox;
   GtkWidget *icon, *label, *line;
+  GtkWidget *dialog_vbox;
+  GtkStyle  *style;
 
   /* remove the main vbox */
-  g_object_ref (G_OBJECT (dialog->vbox));
-  gtk_container_remove (GTK_CONTAINER (dialog), dialog->vbox);
+  dialog_vbox = gtk_bin_get_child (GTK_BIN (dialog));
+  g_object_ref (G_OBJECT (dialog_vbox));
+  gtk_container_remove (GTK_CONTAINER (dialog), dialog_vbox);
 
   /* add a new vbox to the main window */
-  vbox = gtk_vbox_new (FALSE, 0);
+  vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
   gtk_container_add (GTK_CONTAINER (dialog), vbox);
   gtk_widget_show (vbox);
 
   /* event box for the background color */
   ebox = gtk_event_box_new ();
   gtk_box_pack_start (GTK_BOX (vbox), ebox, FALSE, FALSE, 0);
-  gtk_widget_modify_bg (ebox, GTK_STATE_NORMAL, &ebox->style->base[GTK_STATE_NORMAL]);
+  style = gtk_widget_get_style (ebox);
+  gtk_widget_modify_bg (ebox, GTK_STATE_NORMAL, &style->base[GTK_STATE_NORMAL]);
   gtk_widget_show (ebox);
 
   /* create a hbox */
-  hbox = gtk_hbox_new (FALSE, 12);
+  hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 12);
   gtk_container_set_border_width (GTK_CONTAINER (hbox), 6);
   gtk_container_add (GTK_CONTAINER (ebox), hbox);
   gtk_widget_show (hbox);
@@ -462,36 +459,16 @@ mousepad_util_dialog_header (GtkDialog   *dialog,
   g_free (full_title);
 
   /* add the separator between header and content */
-  line = gtk_hseparator_new ();
+  line = gtk_separator_new (GTK_ORIENTATION_HORIZONTAL);
   gtk_box_pack_start (GTK_BOX (vbox), line, FALSE, FALSE, 0);
   gtk_widget_show (line);
 
   /* add the main dialog box to the new vbox */
-  gtk_box_pack_start (GTK_BOX (vbox), GTK_DIALOG (dialog)->vbox, TRUE, TRUE, 0);
-  g_object_unref (G_OBJECT (GTK_DIALOG (dialog)->vbox));
+  gtk_box_pack_start (GTK_BOX (vbox), dialog_vbox, TRUE, TRUE, 0);
+  g_object_unref (G_OBJECT (dialog_vbox));
 }
 
 
-#if !GTK_CHECK_VERSION (2,12,0)
-void
-mousepad_util_set_tooltip (GtkWidget   *widget,
-                           const gchar *string)
-{
-  static GtkTooltips *tooltips = NULL;
-
-  mousepad_return_if_fail (GTK_IS_WIDGET (widget));
-  mousepad_return_if_fail (string ? g_utf8_validate (string, -1, NULL) : TRUE);
-
-  /* allocate the shared tooltips on-demand */
-  if (G_UNLIKELY (tooltips == NULL))
-    tooltips = gtk_tooltips_new ();
-
-  /* setup the tooltip for the widget */
-  gtk_tooltips_set_tip (tooltips, widget, string, NULL);
-}
-#endif
-
-
 gint
 mousepad_util_get_real_line_offset (const GtkTextIter *iter,
                                     gint               tab_size)
@@ -552,7 +529,7 @@ mousepad_util_get_save_location (const gchar *relpath,
 {
   gchar *filename, *dirname;
 
-  mousepad_return_val_if_fail (g_get_user_config_dir () != NULL, NULL);
+  g_return_val_if_fail (g_get_user_config_dir () != NULL, NULL);
 
   /* create the full filename */
   filename = g_build_filename (g_get_user_config_dir (), relpath, NULL);
@@ -664,9 +641,9 @@ mousepad_util_search_iter (const GtkTextIter   *start,
   gboolean     match_case, search_backwards, whole_word;
   guint        needle_offset = 0;
 
-  mousepad_return_val_if_fail (start != NULL, FALSE);
-  mousepad_return_val_if_fail (string != NULL, FALSE);
-  mousepad_return_val_if_fail (limit != NULL, FALSE);
+  g_return_val_if_fail (start != NULL, FALSE);
+  g_return_val_if_fail (string != NULL, FALSE);
+  g_return_val_if_fail (limit != NULL, FALSE);
 
   /* search properties */
   match_case       = (flags & MOUSEPAD_SEARCH_FLAGS_MATCH_CASE) != 0;
@@ -679,9 +656,16 @@ mousepad_util_search_iter (const GtkTextIter   *start,
   /* walk from the start to the end iter */
   for (;;)
     {
-      /* break when we reach the limit iter */
-      if (G_UNLIKELY (gtk_text_iter_equal (&iter, limit)))
-        break;
+      /* break when we reach the limit iter, for backwards searching it's
+       * it has to be less than the limit which is 0 so that we can match
+       * at the very start of the buffer. for forwards searching the limit
+       * is the index after the last char so break when we are at or past
+       * the limit. */
+      if ((search_backwards && gtk_text_iter_compare (&iter, limit) < 0) ||
+          (!search_backwards && gtk_text_iter_compare (&iter, limit) >= 0))
+        {
+          break;
+        }
 
       /* get the unichar characters */
       iter_char = gtk_text_iter_get_char (&iter);
@@ -808,12 +792,12 @@ mousepad_util_search_get_iters (GtkTextBuffer       *buffer,
       else if (flags & (MOUSEPAD_SEARCH_FLAGS_ITER_AREA_END | MOUSEPAD_SEARCH_FLAGS_ITER_SEL_END))
         *iter = sel_end;
       else
-        mousepad_assert_not_reached ();
+        g_assert_not_reached ();
     }
   else
     {
       /* this should never happen */
-      mousepad_assert_not_reached ();
+      g_assert_not_reached ();
     }
 
   /* invert the start and end iter on backwards searching */
@@ -822,6 +806,8 @@ mousepad_util_search_get_iters (GtkTextBuffer       *buffer,
       tmp = *start;
       *start = *end;
       *end = tmp;
+      /* when searching backwards, we need to start before the selection */
+      gtk_text_iter_backward_char (iter);
     }
 }
 
@@ -839,10 +825,10 @@ mousepad_util_highlight (GtkTextBuffer       *buffer,
   gboolean    found, cached = FALSE;
   gint        counter = 0;
 
-  mousepad_return_val_if_fail (GTK_IS_TEXT_BUFFER (buffer), -1);
-  mousepad_return_val_if_fail (GTK_IS_TEXT_TAG (tag), -1);
-  mousepad_return_val_if_fail (string == NULL || g_utf8_validate (string, -1, NULL), -1);
-  mousepad_return_val_if_fail ((flags & MOUSEPAD_SEARCH_FLAGS_DIR_BACKWARD) == 0, -1);
+  g_return_val_if_fail (GTK_IS_TEXT_BUFFER (buffer), -1);
+  g_return_val_if_fail (GTK_IS_TEXT_TAG (tag), -1);
+  g_return_val_if_fail (string == NULL || g_utf8_validate (string, -1, NULL), -1);
+  g_return_val_if_fail ((flags & MOUSEPAD_SEARCH_FLAGS_DIR_BACKWARD) == 0, -1);
 
   /* get the buffer bounds */
   gtk_text_buffer_get_bounds (buffer, &start, &end);
@@ -925,10 +911,10 @@ mousepad_util_search (GtkTextBuffer       *buffer,
   GtkTextIter  match_start, match_end;
   GtkTextMark *mark_start, *mark_iter, *mark_end, *mark_replace;
 
-  mousepad_return_val_if_fail (GTK_IS_TEXT_BUFFER (buffer), -1);
-  mousepad_return_val_if_fail (string && g_utf8_validate (string, -1, NULL), -1);
-  mousepad_return_val_if_fail (replace == NULL || g_utf8_validate (replace, -1, NULL), -1);
-  mousepad_return_val_if_fail ((flags & MOUSEPAD_SEARCH_FLAGS_ACTION_HIGHTLIGHT) == 0, -1);
+  g_return_val_if_fail (GTK_IS_TEXT_BUFFER (buffer), -1);
+  g_return_val_if_fail (string && g_utf8_validate (string, -1, NULL), -1);
+  g_return_val_if_fail (replace == NULL || g_utf8_validate (replace, -1, NULL), -1);
+  g_return_val_if_fail ((flags & MOUSEPAD_SEARCH_FLAGS_ACTION_HIGHTLIGHT) == 0, -1);
 
   /* freeze buffer notifications */
   g_object_freeze_notify (G_OBJECT (buffer));
@@ -972,10 +958,13 @@ mousepad_util_search (GtkTextBuffer       *buffer,
           if (flags & MOUSEPAD_SEARCH_FLAGS_ACTION_SELECT)
             {
               /* select the match */
-              gtk_text_buffer_select_range (buffer, &match_start, &match_end);
+              if (! search_backwards)
+                gtk_text_buffer_select_range (buffer, &match_start, &match_end);
+              else
+                gtk_text_buffer_select_range (buffer, &match_end, &match_start);
             }
           else if (flags & MOUSEPAD_SEARCH_FLAGS_ACTION_REPLACE)
-          {
+            {
               /* create text mark */
               mark_replace = gtk_text_buffer_create_mark (buffer, NULL, &match_start, search_backwards);
 
@@ -1002,8 +991,8 @@ mousepad_util_search (GtkTextBuffer       *buffer,
 
               /* search again */
               search_again = TRUE;
-          }
-        else if (flags & MOUSEPAD_SEARCH_FLAGS_ACTION_NONE)
+            }
+          else if (flags & MOUSEPAD_SEARCH_FLAGS_ACTION_NONE)
             {
               /* keep searching when requested */
               if (flags & MOUSEPAD_SEARCH_FLAGS_ENTIRE_AREA)
@@ -1012,10 +1001,10 @@ mousepad_util_search (GtkTextBuffer       *buffer,
               /* move iter */
               iter = match_end;
             }
-        else
-          {
+          else
+            {
               /* no valid action was defined */
-              mousepad_assert_not_reached ();
+              g_assert_not_reached ();
             }
 
           /* increase the counter */
@@ -1069,138 +1058,108 @@ mousepad_util_search (GtkTextBuffer       *buffer,
 
 
 
-/**
- * Colour scheme functions
- */
-gint
-mousepad_util_color_schemes_name_compare (gconstpointer a,
-                                          gconstpointer b)
+/* get the related action of the widget or walk up the parents to find one.
+ * useful for example to get the related action of a tool item from its child. */
+GtkAction *
+mousepad_util_find_related_action (GtkWidget *widget)
 {
-  const gchar *name_a, *name_b;
+  GtkAction *action;
 
-  if (G_UNLIKELY (!GTK_IS_SOURCE_STYLE_SCHEME (a)))
-    return -(a != b);
-  if (G_UNLIKELY (!GTK_IS_SOURCE_STYLE_SCHEME (b)))
-    return a != b;
+  g_return_val_if_fail (GTK_IS_WIDGET (widget), NULL);
 
-  name_a = gtk_source_style_scheme_get_name (GTK_SOURCE_STYLE_SCHEME (a));
-  name_b = gtk_source_style_scheme_get_name (GTK_SOURCE_STYLE_SCHEME (b));
-
-  return g_utf8_collate (name_a, name_b);
-}
-
-
-
-GSList *
-mousepad_util_color_schemes_get (void)
-{
-  GSList               *list = NULL;
-  const gchar * const  *schemes;
-  GtkSourceStyleScheme *scheme;
-
-  schemes = gtk_source_style_scheme_manager_get_scheme_ids (
-              gtk_source_style_scheme_manager_get_default ());
-
-  while (*schemes)
+  do
     {
-      scheme = gtk_source_style_scheme_manager_get_scheme (
-                gtk_source_style_scheme_manager_get_default (), *schemes);
-      list = g_slist_prepend (list, scheme);
-      schemes++;
-    }
+      if (GTK_IS_ACTIVATABLE (widget))
+        action = gtk_activatable_get_related_action (GTK_ACTIVATABLE (widget));
 
-  return list;
-}
-
-
-
-GSList *
-mousepad_util_color_schemes_get_sorted (void)
-{
-  return g_slist_sort (mousepad_util_color_schemes_get (),
-                       mousepad_util_color_schemes_name_compare);
-}
-
-
-
-/**
- * Language/filetype functions
- */
-gint
-mousepad_util_languages_name_compare (gconstpointer a,
-                                      gconstpointer b)
-{
-  const gchar *name_a, *name_b;
-
-  if (G_UNLIKELY (!GTK_IS_SOURCE_LANGUAGE (a)))
-    return -(a != b);
-  if (G_UNLIKELY (!GTK_IS_SOURCE_LANGUAGE (b)))
-    return a != b;
-
-  name_a = gtk_source_language_get_name (GTK_SOURCE_LANGUAGE (a));
-  name_b = gtk_source_language_get_name (GTK_SOURCE_LANGUAGE (b));
-
-  return g_utf8_collate (name_a, name_b);
-}
-
-
-
-GSList *
-mousepad_util_language_sections_get_sorted (void)
-{
-  GSList                   *list = NULL;
-  const gchar *const       *languages;
-  GtkSourceLanguage        *language;
-  GtkSourceLanguageManager *manager;
-
-  manager = gtk_source_language_manager_get_default ();
-  languages = gtk_source_language_manager_get_language_ids (manager);
-
-  while (*languages)
-    {
-      language = gtk_source_language_manager_get_language (manager, *languages);
-      if (G_LIKELY (GTK_IS_SOURCE_LANGUAGE (language)))
+      if (G_UNLIKELY (action == NULL))
         {
-          /* ensure no duplicates in list */
-          if (!g_slist_find_custom (list,
-                                    gtk_source_language_get_section (language),
-                                    (GCompareFunc)g_strcmp0))
-            {
-              list = g_slist_prepend (list, (gchar *)gtk_source_language_get_section (language));
-            }
+          if (gtk_widget_is_toplevel (widget))
+            break;
+          widget = gtk_widget_get_parent (widget);
         }
-      languages++;
     }
+  while (action == NULL);
 
-  return g_slist_sort (list, (GCompareFunc)g_strcmp0);
+  return action;
 }
 
 
 
-GSList *
-mousepad_util_languages_get_sorted_for_section (const gchar *section)
+GIcon *
+mousepad_util_icon_for_mime_type (const gchar *mime_type)
 {
-  GSList                   *list = NULL;
-  const gchar *const       *languages;
-  GtkSourceLanguage        *language;
-  GtkSourceLanguageManager *manager;
+  gchar *content_type;
+  GIcon *icon = NULL;
 
-  mousepad_return_val_if_fail (section != NULL, NULL);
+  g_return_val_if_fail (mime_type != NULL, NULL);
 
-  manager = gtk_source_language_manager_get_default ();
-  languages = gtk_source_language_manager_get_language_ids (manager);
+  content_type = g_content_type_from_mime_type (mime_type);
+  if (content_type != NULL)
+    icon = g_content_type_get_icon (content_type);
 
-  while (*languages)
+  return icon;
+}
+
+
+
+static void
+mousepad_util_container_foreach_counter (GtkWidget *widget,
+                                         guint     *n_children)
+{
+  *n_children = *n_children + 1;
+}
+
+
+
+gboolean
+mousepad_util_container_has_children (GtkContainer *container)
+{
+  guint n_children = 0;
+
+  g_return_val_if_fail (GTK_IS_CONTAINER (container), FALSE);
+
+  gtk_container_foreach (container,
+                         (GtkCallback) mousepad_util_container_foreach_counter,
+                         &n_children);
+
+  return (n_children > 0);
+}
+
+
+void
+mousepad_util_container_clear (GtkContainer *container)
+{
+  GList *list, *iter;
+
+  g_return_if_fail (GTK_IS_CONTAINER (container));
+
+  list = gtk_container_get_children (container);
+
+  for (iter = list; iter != NULL; iter = g_list_next (iter))
+    gtk_container_remove (container, iter->data);
+
+  g_list_free (list);
+}
+
+
+
+void
+mousepad_util_container_move_children (GtkContainer *source,
+                                       GtkContainer *destination)
+{
+  GList *list, *iter;
+
+  list = gtk_container_get_children (source);
+
+  for (iter = list; iter != NULL; iter = g_list_next (iter))
     {
-      language = gtk_source_language_manager_get_language (manager, *languages);
-      if (G_LIKELY (GTK_IS_SOURCE_LANGUAGE (language)))
-        {
-          /* only get languages in the specified section */
-          if (g_strcmp0 (gtk_source_language_get_section (language), section) == 0)
-            list = g_slist_prepend (list, language);
-        }
-      languages++;
+      GtkWidget *tmp = g_object_ref (iter->data);
+
+      gtk_container_remove (source, tmp);
+      gtk_container_add (destination, tmp);
+      g_object_unref (tmp);
     }
 
-  return g_slist_sort(list, (GCompareFunc)mousepad_util_languages_name_compare);
+  g_list_free (list);
 }
